@@ -51,19 +51,20 @@ class LocalCluster(Thread):
     client: ClientThread
 
     def __init__(self,
-                 source: Iterable[str] = None, template: str = DEFAULT_TEMPLATE,
-                 forever_mode: bool = False, restart_mode: bool = False,
+                 source: Iterable[str] = None, num_tasks: int = 1, template: str = DEFAULT_TEMPLATE,
                  bundlesize: int = DEFAULT_BUNDLESIZE, bundlewait: int = DEFAULT_BUNDLEWAIT,
-                 max_retries: int = DEFAULT_ATTEMPTS, eager: bool = False, live: bool = False,
-                 num_tasks: int = 1, redirect_failures: IO = None,
-                 redirect_output: IO = None, redirect_errors: IO = None,
+                 in_memory: bool = False, no_confirm: bool = False,
+                 forever_mode: bool = False, restart_mode: bool = False,
+                 max_retries: int = DEFAULT_ATTEMPTS, eager: bool = False,
+                 redirect_failures: IO = None, redirect_output: IO = None, redirect_errors: IO = None,
                  delay_start: float = DEFAULT_DELAY, capture: bool = False) -> None:
         """Initialize server and client threads."""
         auth = secrets.token_hex(64)
-        self.server = ServerThread(source=source, auth=auth, live=live, bundlesize=bundlesize, bundlewait=bundlewait,
+        self.server = ServerThread(source=source, auth=auth, in_memory=in_memory, no_confirm=no_confirm,
+                                   bundlesize=bundlesize, bundlewait=bundlewait,
                                    max_retries=max_retries, eager=eager, forever_mode=forever_mode,
                                    restart_mode=restart_mode, redirect_failures=redirect_failures)
-        self.client = ClientThread(num_tasks=num_tasks, template=template, auth=auth,
+        self.client = ClientThread(num_tasks=num_tasks, template=template, auth=auth, no_confirm=no_confirm,
                                    bundlesize=bundlesize, bundlewait=bundlewait, delay_start=delay_start,
                                    redirect_output=redirect_output, redirect_errors=redirect_errors,
                                    capture=capture)
@@ -92,24 +93,25 @@ class RemoteCluster(Thread):
     client_argv: str
 
     def __init__(self,
-                 source: Iterable[str] = None, template: str = DEFAULT_TEMPLATE,
-                 forever_mode: bool = False, restart_mode: bool = False,
-                 launcher: str = 'mpirun', launcher_args: List[str] = None,
-                 bind: Tuple[str, int] = ('0.0.0.0', QueueConfig.port),
+                 source: Iterable[str] = None, num_tasks: int = 1, template: str = DEFAULT_TEMPLATE,
                  bundlesize: int = DEFAULT_BUNDLESIZE, bundlewait: int = DEFAULT_BUNDLEWAIT,
-                 max_retries: int = DEFAULT_ATTEMPTS, eager: bool = False, live: bool = False,
-                 num_tasks: int = 1, remote_exe: str = 'hyper-shell', redirect_failures: IO = None,
-                 delay_start: float = DEFAULT_DELAY, capture: bool = False) -> None:
+                 forever_mode: bool = False, restart_mode: bool = False,
+                 bind: Tuple[str, int] = ('0.0.0.0', QueueConfig.port), delay_start: float = DEFAULT_DELAY,
+                 launcher: str = 'mpirun', launcher_args: List[str] = None, remote_exe: str = 'hyper-shell',
+                 max_retries: int = DEFAULT_ATTEMPTS, eager: bool = False, redirect_failures: IO = None,
+                 in_memory: bool = False, no_confirm: bool = False, capture: bool = False) -> None:
         """Initialize server and client threads."""
         auth = secrets.token_hex(64)
-        self.server = ServerThread(source=source, auth=auth, live=live, bundlesize=bundlesize,
-                                   bundlewait=bundlewait, max_retries=max_retries, eager=eager, address=bind,
-                                   forever_mode=forever_mode, restart_mode=restart_mode,
+        self.server = ServerThread(source=source, auth=auth, bundlesize=bundlesize, bundlewait=bundlewait,
+                                   in_memory=in_memory, no_confirm=no_confirm, max_retries=max_retries, eager=eager,
+                                   address=bind, forever_mode=forever_mode, restart_mode=restart_mode,
                                    redirect_failures=redirect_failures)
         launcher_args = '' if launcher_args is None else ' '.join(launcher_args)
         client_args = ''
         if capture is True:
             client_args += ' --capture'
+        if no_confirm is True:
+            client_args += ' --no-confirm'
         self.client_argv = (f'{launcher} {launcher_args} {remote_exe} client -H {HOSTNAME} -p {bind[1]} '
                             f'-N {num_tasks} -b {bundlesize} -w {bundlewait} -t "{template}" -k {auth} '
                             f'-d {delay_start} {client_args}')
@@ -135,7 +137,7 @@ class RemoteCluster(Thread):
 class NodeList(list):
     """A list of hostnames."""
 
-    name_pattern: re.Pattern = re.compile(r'^[a-z-A-Z0-9]*$')
+    group_pattern: re.Pattern = re.compile(r',(?![^[]*])')
     range_pattern: re.Pattern = re.compile(r'\[((\d+|\d+-\d+)(,(\d+|\d+-\d+))*)]')
 
     @classmethod
@@ -143,7 +145,7 @@ class NodeList(list):
         """Smart initialization via some command-line `arg`."""
         if not arg:
             return cls.from_config()
-        if cls.range_pattern.search(arg):
+        if ',' in arg or cls.range_pattern.search(arg):
             return cls.from_pattern(arg)
         else:
             return cls([arg, ])
@@ -179,13 +181,18 @@ class NodeList(list):
     @classmethod
     def from_pattern(cls: Type[NodeList], pattern: str) -> NodeList:
         """Expand a `pattern` to multiple hostnames."""
-        if match := cls.range_pattern.search(pattern):
-            range_spec = match.group()
-            prefix, suffix = pattern.split(range_spec)
-            segments = cls.expand_pattern(range_spec)
-            return cls([f'{prefix}{segment}{suffix}' for segment in segments])
+        pattern = pattern.strip(',')
+        if match := cls.group_pattern.search(pattern):
+            idx = match.start()
+            return cls(cls.from_pattern(pattern[0:idx]) + cls.from_pattern(pattern[idx+1:]))
         else:
-            return cls([pattern, ])
+            if match := cls.range_pattern.search(pattern):
+                range_spec = match.group()
+                prefix, suffix = pattern.split(range_spec)
+                segments = cls.expand_pattern(range_spec)
+                return cls([f'{prefix}{segment}{suffix}' for segment in segments])
+            else:
+                return cls([pattern, ])
 
     @staticmethod
     def expand_pattern(spec: str) -> List[str]:
@@ -218,21 +225,22 @@ class SSHCluster(Thread):
     client_argv: List[str]
 
     def __init__(self,
-                 source: Iterable[str] = None, template: str = DEFAULT_TEMPLATE,
+                 source: Iterable[str] = None, num_tasks: int = 1, template: str = DEFAULT_TEMPLATE,
                  forever_mode: bool = False, restart_mode: bool = False,
-                 bind: Tuple[str, int] = ('0.0.0.0', QueueConfig.port),
+                 bind: Tuple[str, int] = ('0.0.0.0', QueueConfig.port), remote_exe: str = 'hyper-shell',
                  launcher: str = 'ssh', launcher_args: List[str] = None, nodelist: List[str] = None,
                  bundlesize: int = DEFAULT_BUNDLESIZE, bundlewait: int = DEFAULT_BUNDLEWAIT,
-                 max_retries: int = DEFAULT_ATTEMPTS, eager: bool = False, live: bool = False,
-                 num_tasks: int = 1, remote_exe: str = 'hyper-shell', redirect_failures: IO = None,
-                 export_env: bool = False, delay_start: float = DEFAULT_DELAY, capture: bool = False) -> None:
+                 max_retries: int = DEFAULT_ATTEMPTS, eager: bool = False, in_memory: bool = False,
+                 no_confirm: bool = False, redirect_failures: IO = None, export_env: bool = False,
+                 delay_start: float = DEFAULT_DELAY, capture: bool = False) -> None:
         """Initialize server and client threads."""
         if nodelist is None:
             raise AttributeError('Expected nodelist')
         auth = secrets.token_hex(64)
-        self.server = ServerThread(source=source, auth=auth, live=live, bundlesize=bundlesize,
-                                   bundlewait=bundlewait, max_retries=max_retries, eager=eager, address=bind,
+        self.server = ServerThread(source=source, auth=auth, bundlesize=bundlesize, bundlewait=bundlewait,
+                                   max_retries=max_retries, eager=eager, address=bind,
                                    forever_mode=forever_mode, restart_mode=restart_mode,
+                                   in_memory=in_memory, no_confirm=no_confirm,
                                    redirect_failures=redirect_failures)
         launcher_env = '' if not export_env else compile_env()
         if launcher_args is None:
@@ -242,6 +250,8 @@ class SSHCluster(Thread):
         client_args = ''
         if capture is True:
             client_args += ' --capture'
+        if no_confirm:
+            client_args += ' --no-confirm'
         self.client_argv = [f'{launcher} {launcher_args} {host} {launcher_env} {remote_exe} '
                             f'client -H {HOSTNAME} -p {bind[1]} -N {num_tasks} -b {bundlesize} -w {bundlewait} '
                             f'-t \'"{template}"\' -k {auth} -d {delay_start} {client_args}'
@@ -302,9 +312,9 @@ APP_NAME = 'hyper-shell cluster'
 APP_USAGE = f"""\
 Usage:
 hyper-shell cluster [-h] [FILE | --restart | --forever] [-N NUM] [-t CMD] [-b SIZE] [-w SEC]
-                    [-r NUM [--eager]] [-f PATH] [--capture | [-o PATH] [-e PATH]] [--delay-start SEC] 
+                    [-p PORT] [-r NUM [--eager]] [-f PATH] [--capture | [-o PATH] [-e PATH]]
                     [--ssh [HOST... | --ssh-group NAME] [--env] | --mpi | --launcher=ARGS...]
-                    [--no-db | --initdb]
+                    [--no-db | --initdb] [--no-confirm] [--delay-start SEC]
 
 Start cluster locally, over SSH, or with a custom launcher.\
 """
@@ -330,6 +340,7 @@ Options:
       --eager                 Schedule failed tasks before new tasks.
       --no-db                 Disable database (submit directly to clients).
       --initdb                Auto-initialize database.
+      --no-confirm            Disable client confirmation of task bundle received.
       --forever               Schedule forever.
       --restart               Start scheduling from last completed task.
       --ssh-args     ARGS     Command-line arguments for SSH.
@@ -376,12 +387,15 @@ class ClusterApp(Application):
     interface.add_argument('-r', '--max-retries', type=int, default=max_retries)
     interface.add_argument('--eager', action='store_true', dest='eager_mode')
 
-    live_mode: bool = False
+    in_memory: bool = False
     auto_initdb: bool = False
     db_interface = interface.add_mutually_exclusive_group()
-    db_interface.add_argument('--no-db', action='store_true', dest='live_mode')
+    db_interface.add_argument('--no-db', action='store_true', dest='in_memory')
     db_interface.add_argument('--initdb', action='store_true', dest='auto_initdb')
 
+    no_confirm: bool = False
+    interface.add_argument('--no-confirm', action='store_true')
+
     forever_mode: bool = False
     interface.add_argument('--forever', action='store_true', dest='forever_mode')
 
@@ -425,11 +439,10 @@ class ClusterApp(Application):
         """Run cluster."""
         launcher = self.launchers.get(self.mode)
         launcher(source=self.source, num_tasks=self.num_tasks, template=self.template,
-                 bundlesize=self.bundlesize, bundlewait=self.bundlewait,
-                 max_retries=self.max_retries, live=self.live_mode,
-                 forever_mode=self.forever_mode, restart_mode=self.restart_mode,
-                 redirect_failures=self.failure_stream, delay_start=self.delay_start,
-                 capture=self.capture)
+                 bundlesize=self.bundlesize, bundlewait=self.bundlewait, max_retries=self.max_retries,
+                 in_memory=self.in_memory, no_confirm=self.no_confirm, forever_mode=self.forever_mode,
+                 restart_mode=self.restart_mode, redirect_failures=self.failure_stream,
+                 delay_start=self.delay_start, capture=self.capture)
 
     def run_local(self, **options) -> None:
         """Run local cluster."""
@@ -475,7 +488,7 @@ class ClusterApp(Application):
 
     def check_arguments(self) -> None:
         """Various checks on input arguments."""
-        if self.restart_mode and self.live_mode:
+        if self.restart_mode and self.in_memory:
             raise ArgumentError('Cannot restart without database (given --no-db)')
         if self.filepath is None and not self.restart_mode:
             self.filepath = '-'  # NOTE: assume STDIN
@@ -487,9 +500,9 @@ class ClusterApp(Application):
             raise ArgumentError('Cannot specify -c/--capture with -o/--output')
         if self.capture and self.errors_path:
             raise ArgumentError('Cannot specify -c/--capture with -e/--error')
-        if self.live_mode and self.forever_mode:
+        if self.in_memory and self.forever_mode:
             raise ArgumentError('Using --forever with --no-db is invalid')
-        if self.live_mode and self.restart_mode:
+        if self.in_memory and self.restart_mode:
             raise ArgumentError('Using --restart with --no-db is invalid')
         if self.forever_mode and self.restart_mode:
             raise ArgumentError('Using --forever with --restart is invalid')
@@ -531,7 +544,7 @@ class ClusterApp(Application):
         self.check_arguments()
         if config.database.provider == 'sqlite' or self.auto_initdb:
             initdb()  # Auto-initialize if local sqlite provider
-        elif not self.live_mode:
+        elif not self.in_memory:
             checkdb()
         return self
 
